@@ -83,11 +83,11 @@ the function is called with the same input parameters the result is retrieved
 from the cache and not recomputed. There are many implementations of 
 ``memoize`` in http://www.python.org/moin/PythonDecoratorLibrary, 
 but they do not preserve the signature.
-A simple implementation for Python 2.5 could be the following (notice
+A simple implementation could be the following (notice
 that in general it is impossible to memoize correctly something
 that depends on non-hashable arguments):
 
-$$memoize25
+$$memoize_uw
 
 Here we used the functools.update_wrapper_ utility, which has
 been added in Python 2.5 expressly to simplify the definition of decorators
@@ -100,14 +100,14 @@ from the original function to the decorated function by hand).
 The implementation above works in the sense that the decorator 
 can accept functions with generic signatures; unfortunately this
 implementation does *not* define a signature-preserving decorator, since in
-general ``memoize25`` returns a function with a
+general ``memoize_uw`` returns a function with a
 *different signature* from the original function.
 
 Consider for instance the following case:
 
 .. code-block:: python
 
- >>> @memoize25
+ >>> @memoize_uw
  ... def f1(x):
  ...     time.sleep(1) # simulate some long computation
  ...     return x
@@ -119,8 +119,8 @@ keyword arguments:
 .. code-block:: python
 
  >>> from inspect import getargspec 
- >>> print getargspec(f1) 
- ([], 'args', 'kw', None)
+ >>> print getargspec(f1) # I am using Python 2.6+ here
+ ArgSpec(args=[], varargs='args', keywords='kw', defaults=None)
 
 This means that introspection tools such as pydoc will give
 wrong informations about the signature of ``f1``. This is pretty bad:
@@ -160,7 +160,7 @@ At this point you can define your decorator as follows:
 
 $$memoize
 
-The difference with respect to the Python 2.5 approach, which is based
+The difference with respect to the ``memoize_uw`` approach, which is based
 on nested functions, is that the decorator module forces you to lift
 the inner function at the outer level (*flat is better than nested*).
 Moreover, you are forced to pass explicitly the function you want to
@@ -186,7 +186,7 @@ The signature of ``heavy_computation`` is the one you would expect:
 .. code-block:: python
 
  >>> print getargspec(heavy_computation) 
- ([], None, None, None)
+ ArgSpec(args=[], varargs=None, keywords=None, defaults=None)
 
 A ``trace`` decorator
 ------------------------------------------------------
@@ -219,7 +219,7 @@ and it that it has the correct signature:
 .. code-block:: python
 
  >>> print getargspec(f1) 
- (['x'], None, None, None)
+ ArgSpec(args=['x'], varargs=None, keywords=None, defaults=None)
 
 The same decorator works with functions of any signature:
 
@@ -233,7 +233,7 @@ The same decorator works with functions of any signature:
  calling f with args (0, 3, 2), {}
  
  >>> print getargspec(f) 
- (['x', 'y', 'z'], 'args', 'kw', (1, 2))
+ ArgSpec(args=['x', 'y', 'z'], varargs='args', keywords='kw', defaults=(1, 2))
 
 That includes even functions with exotic signatures like the following:
 
@@ -243,7 +243,7 @@ That includes even functions with exotic signatures like the following:
  ... def exotic_signature((x, y)=(1,2)): return x+y
  
  >>> print getargspec(exotic_signature)
- ([['x', 'y']], None, None, ((1, 2),))
+ ArgSpec(args=[['x', 'y']], varargs=None, keywords=None, defaults=((1, 2),))
  >>> exotic_signature() 
  calling exotic_signature with args ((1, 2),), {}
  3
@@ -275,16 +275,15 @@ For instance, you can write directly
  ...     print "calling %s with args %s, %s" % (f.func_name, args, kw)
  ...     return f(*args, **kw)
 
-and now ``trace`` will be a decorator. You
-can easily check that the signature has changed:
+and now ``trace`` will be a decorator. Actually ``trace`` is a ``partial``
+object which can be used as a decorator:
 
 .. code-block:: python
 
- >>> print getargspec(trace)
- (['f'], None, None, None)
+ >>> trace # doctest: +ELLIPSIS
+ <function trace at 0x...>
 
-Therefore now ``trace`` can be used as a decorator and
-the following will work:
+Here is an example of usage:
 
 .. code-block:: python
 
@@ -294,21 +293,19 @@ the following will work:
  >>> func()
  calling func with args (), {}
 
-For the rest of this document, I will discuss examples of useful
-decorators built on top of ``decorator``.
+If you are using an old Python version (Python 2.4) the
+``decorator`` module provides a poor man replacement for
+``functools.partial``.
 
 ``blocking``
 -------------------------------------------
 
 Sometimes one has to deal with blocking resources, such as ``stdin``, and
 sometimes it is best to have back a "busy" message than to block everything. 
-This behavior can be implemented with a suitable decorator:
+This behavior can be implemented with a suitable family of decorators,
+where the parameter is the busy message:
 
 $$blocking
-
-(notice that without the help of ``decorator``, an additional level of 
-nesting would have been needed). This is actually an example
-of a one-parameter family of decorators.
    
 Functions decorated with ``blocking`` will return a busy message if
 the resource is unavailable, and the intended result if the resource is 
@@ -387,18 +384,20 @@ be locked. Here is a minimalistic example:
 Each call to ``write`` will create a new writer thread, but there will 
 be no synchronization problems since ``write`` is locked.
 
->>> write("data1") 
-<Thread(write-1, started)>
+.. code-block:: python
 
->>> time.sleep(.1) # wait a bit, so we are sure data2 is written after data1
-
->>> write("data2") 
-<Thread(write-2, started)>
-
->>> time.sleep(2) # wait for the writers to complete
-
->>> print datalist
-['data1', 'data2']
+ >>> write("data1") # doctest: +ELLIPSIS
+ <Thread(write-1, started...)>
+ 
+ >>> time.sleep(.1) # wait a bit, so we are sure data2 is written after data1
+ 
+ >>> write("data2") # doctest: +ELLIPSIS
+ <Thread(write-2, started...)>
+ 
+ >>> time.sleep(2) # wait for the writers to complete
+ 
+ >>> print datalist
+ ['data1', 'data2']
 
 The ``FunctionMaker`` class
 ---------------------------------------------------------------
@@ -409,9 +408,62 @@ a ``FunctionMaker`` class which is able to generate on the fly
 functions with a given name and signature from a function template
 passed as a string. Generally speaking, you should not need to
 resort to ``FunctionMaker`` when writing ordinary decorators, but
-it is handy in some circumstances. We will see an example in two
-paragraphs, when implementing a custom decorator factory
-(``decorator_apply``).
+it is handy in some circumstances. You will see an example shortly, in
+the implementation of a cool decorator utility (``decorator_apply``).
+
+``FunctionMaker`` provides a ``.create`` classmethod which
+takes as input the name, signature, and body of the function
+we want to generate as well as the execution environment
+were the function is generated by ``exec``. Here is an example:
+
+.. code-block:: python
+
+ >>> def f(*args, **kw): # a function with a generic signature
+ ...     print args, kw
+
+ >>> f1 = FunctionMaker.create('f1(a, b)', 'f(a, b)', dict(f=f))
+ >>> f1(1,2)
+ (1, 2) {}
+
+It is important to notice that the function body is interpolated
+before being executed, so be careful with the ``%`` sign!
+
+``FunctionMaker.create`` also accepts keyword arguments and such
+arguments are attached to the resulting function. This is useful
+if you want to set some function attributes, for instance the
+docstring ``__doc__``.
+
+For debugging/introspection purposes it may be useful to see
+the source code of the generated function; to do that, just
+pass the flag ``addsource=True`` and a ``__source__`` attribute will
+be added to the generated function:
+
+.. code-block:: python
+
+ >>> f1 = FunctionMaker.create(
+ ...     'f1(a, b)', 'f(a, b)', dict(f=f), addsource=True)
+ >>> print f1.__source__
+ def f1(a, b):
+     f(a, b)
+ <BLANKLINE>
+
+``FunctionMaker.create`` can take as first argument a string,
+as in the examples before, or a function. This is the most common
+usage, since typically you want to decorate a pre-existing
+function. A framework author may want to use directly ``FunctionMaker.create``
+instead of ``decorator``, since it gives you direct access to the body
+of the generated function. For instance, suppose you want to instrument
+the ``__init__`` methods of a set of classes, by preserving their
+signature (such use case is not made up; this is done in SQAlchemy
+and in other frameworks). When the first argument of ``FunctionMaker.create``
+is a function, a ``FunctionMaker`` object is instantiated internally,
+with attributes ``args``, ``varargs``,
+``keywords`` and ``defaults`` which are the
+the return values of the standard library function ``inspect.getargspec``.
+For each argument in the ``args`` (which is a list of strings containing
+the names of the mandatory arguments) an attribute ``arg0``, ``arg1``,
+..., ``argN`` is also generated. Finally, there is a ``signature`` 
+attribute, a string with the signature of the original function.
 
 Notice that while I do not have plans
 to change or remove the functionality provided in the
@@ -424,57 +476,10 @@ On the other hand, the functionality provided by
 ``decorator`` has been there from version 0.1 and it is guaranteed to
 stay there forever.
 
-``FunctionMaker`` takes the name and the signature (as a string) of a
-function in input, or a whole function. Then, it creates a new
-function (actually a closure) from a function template (the function
-template must begin with ``def`` with no comments before and you cannot
-use a ``lambda``) via its
-``.make`` method: the name and the signature of the resulting function
-are determinated by the specified name and signature.  For instance,
-here is an example of how to restrict the signature of a function:
-
-.. code-block:: python
-
- >>> def f(*args, **kw): # a function with a generic signature
- ...     print args, kw
-
- >>> fun = FunctionMaker(name="f1", signature="a,b")
- >>> f1 = fun.make('''\
- ... def %(name)s(%(signature)s):
- ...     f(%(signature)s)''', dict(f=f))
- ...
- >>> f1(1,2)
- (1, 2) {}
-
-The dictionary passed in this example (``dict(f=f)``) is the
-execution environment: ``FunctionMaker.make`` actually returns a
-closure, and the original function ``f`` is a variable in the
-closure environment.
-``FunctionMaker.make`` also accepts keyword arguments and such
-arguments are attached to the resulting function. This is useful
-if you want to set some function attributes, for instance the
-docstring ``__doc__``.
-
-For debugging/introspection purposes it may be useful to see
-the source code of the generated function; to do that, just
-pass the flag ``addsource=True`` and a ``__source__`` attribute will
-be added to the decorated function:
-
-.. code-block:: python
-
- >>> f1 = fun.make('''\
- ... def %(name)s(%(signature)s):
- ...     f(%(signature)s)''', dict(f=f), addsource=True)
- ...
- >>> print f1.__source__
- def f1(a,b):
-     f(a,b)
- <BLANKLINE>
-
 Getting the source code
 ---------------------------------------------------
 
-Internally ``FunctionMaker.make`` uses ``exec`` to generate the
+Internally ``FunctionMaker.create`` uses ``exec`` to generate the
 decorated function. Therefore
 ``inspect.getsource`` will not work for decorated functions. That
 means that the usual '??' trick in IPython will give you the (right on
@@ -497,16 +502,16 @@ $$identity_dec
  <BLANKLINE>
 
 (see bug report 1764286_ for an explanation of what is happening).
-Unfortunately the bug is still there, even in Python 2.6 and 3.0.
+Unfortunately the bug is still there, even in Python 2.7 and 3.1.
 There is however a workaround. The decorator module adds an
-attribute ``.undecorated`` to the decorated function, containing
+attribute ``.__wrapped__`` to the decorated function, containing
 a reference to the original function. The easy way to get
 the source code is to call ``inspect.getsource`` on the
 undecorated function:
 
 .. code-block:: python
 
- >>> print inspect.getsource(factorial.undecorated)
+ >>> print inspect.getsource(factorial.__wrapped__)
  @tail_recursive
  def factorial(n, acc=1):
      "The good old factorial"
@@ -528,7 +533,7 @@ having to rewrite them in terms of ``decorator``. You can use a
 
 $$decorator_apply
 
-``decorator_apply`` sets the attribute ``.undecorated`` of the generated
+``decorator_apply`` sets the attribute ``.__wrapped__`` of the generated
 function to the original function, so that you can get the right
 source code.
 
@@ -647,9 +652,10 @@ would require to change the CPython implementation of functions and
 add an hook to make it possible to change their signature directly. 
 That could happen in future versions of Python (see PEP 362_) and 
 then the decorator module would become obsolete. However, at present,
-even in Python 3.0 it is impossible to change the function signature
+even in Python 3.1 it is impossible to change the function signature
 directly, therefore the ``decorator`` module is still useful.
-Actually, this is one of the main reasons why I am releasing version 3.0.
+Actually, this is one of the main reasons why I keep maintaining
+the module and releasing new versions.
 
 .. _362: http://www.python.org/dev/peps/pep-0362
 
@@ -661,27 +667,33 @@ that you can decorate a method, but only before if becomes a bound or unbound
 method, i.e. inside the class.
 Here is an example of valid decoration:
 
->>> class C(object): 
-...      @trace
-...      def meth(self):
-...          pass
+.. code-block:: python
+
+ >>> class C(object): 
+ ...      @trace
+ ...      def meth(self):
+ ...          pass
 
 Here is an example of invalid decoration, when the decorator in
 called too late:
 
->>> class C(object):
-...      def meth(self):
-...          pass
-...
->>> trace(C.meth)
-Traceback (most recent call last):
-  ...
-TypeError: You are decorating a non function: <unbound method C.meth>
+.. code-block:: python
+
+ >>> class C(object):
+ ...      def meth(self):
+ ...          pass
+ ...
+ >>> trace(C.meth)
+ Traceback (most recent call last):
+   ...
+ TypeError: You are decorating a non function: <unbound method C.meth>
 
 The solution is to extract the inner function from the unbound method:
 
->>> trace(C.meth.im_func) # doctest: +ELLIPSIS
-<function meth at 0x...>
+.. code-block:: python
+
+ >>> trace(C.meth.im_func) # doctest: +ELLIPSIS
+ <function meth at 0x...>
 
 There is a restriction on the names of the arguments: for instance,
 if try to call an argument ``_call_`` or ``_func_``
@@ -698,7 +710,9 @@ you will get a ``NameError``:
  def f(_func_):
      return _call_(_func_, _func_)
 
-Finally, the implementation is such that the decorated function contains
+Finally, the implementation is such that the decorated function
+attribute ``.func_globals`` is a *copy* of the original function
+attribute. Moreover the decorated function contains
 a *copy* of the original function dictionary
 (``vars(decorated_f) is not vars(f)``):
 
@@ -719,60 +733,63 @@ a *copy* of the original function dictionary
 Compatibility notes
 ---------------------------------------------------------------
 
-Version 3.0 is a complete rewrite of the original implementation.
-It is mostly compatible with the past, a part for a few differences.
+Version 3.3 is the first version of the ``decorator`` module to fully
+support Python 3, including `function annotations`_. Version 3.2 was the
+first version to support Python 3 via the ``2to3`` conversion tool
+invoked in the build process by the distribute_ project, the Python
+3-compatible replacement of easy_install.  The hard work (for me) has
+been converting the documentation and the doctests.  This has been
+possible only after that docutils_ and pygments_ have been ported to
+Python 3.
 
-First of all, the utilites ``get_info`` and ``new_wrapper``, available
-in the 2.X versions, have been deprecated and they will be removed
-in the future. For the moment, using them raises a ``DeprecationWarning``.
-Incidentally, the functionality has been implemented through a
-decorator which makes a good example for this documentation:
+Version 3 of the ``decorator`` module do not contain any backward
+incompatible change, apart from the removal of the functions
+``get_info`` and ``new_wrapper``, which have been deprecated for
+years. ``get_info`` has been removed since it was little used and
+since it had to be changed anyway to work with Python 3.0;
+``new_wrapper`` has been removed since it was useless: its major use
+case (converting signature changing decorators to signature preserving
+decorators) has been subsumed by ``decorator_apply``, whereas the other use
+case can be managed with the ``FunctionMaker``.
 
-$$deprecated
-
-``get_info`` has been removed since it was little used and since it had
-to be changed anyway to work with Python 3.0; ``new_wrapper`` has been
-removed since it was useless: its major use case (converting
-signature changing decorators to signature preserving decorators)
-has been subsumed by ``decorator_apply``
-and the other use case can be managed with the ``FunctionMaker``.
+There are a few changes in the documentation: I removed the 
+``decorator_factory`` example, which was confusing some of my users,
+and I removed the part about exotic signatures in the Python 3
+documentation, since Python 3 does not support them.
 
 Finally ``decorator`` cannot be used as a class decorator and the
 `functionality introduced in version 2.3`_ has been removed. That
 means that in order to define decorator factories with classes you
 need to define the ``__call__`` method explicitly (no magic anymore).
-
 All these changes should not cause any trouble, since they were
 all rarely used features. Should you have any trouble, you can always
 downgrade to the 2.3 version.
 
-The examples shown here have been tested with Python 2.5. Python 2.4
+The examples shown here have been tested with Python 2.6. Python 2.4
 is also supported - of course the examples requiring the ``with``
-statement will not work there. Python 2.6 works fine, but if you
-run the examples here in the interactive interpreter
-you will notice a couple of minor differences since
+statement will not work there. Python 2.5 works fine, but if you
+run the examples in the interactive interpreter
+you will notice a few differences since
 ``getargspec`` returns an ``ArgSpec`` namedtuple instead of a regular
-tuple, and the string representation of a thread object returns a
-thread identifier number. That means that running the file
-``documentation.py`` under Python 2.5 will a few errors, but
-they are not serious. Python 3.0 is kind of supported too.
-Simply run the script ``2to3`` on the module
-``decorator.py`` and you will get a version of the code running
-with Python 3.0 (at least, I did some simple checks and it seemed
-to work). However there is no support for `function annotations`_ yet
-since it seems premature at this moment (most people are
-still using Python 2.5).
+tuple. That means that running the file
+``documentation.py`` under Python 2.5 will print a few errors, but
+they are not serious. 
 
 .. _functionality introduced in version 2.3: http://www.phyast.pitt.edu/~micheles/python/documentation.html#class-decorators-and-decorator-factories
 .. _function annotations: http://www.python.org/dev/peps/pep-3107/
-
+.. _distribute: http://packages.python.org/distribute/
+.. _docutils: http://docutils.sourceforge.net/
+.. _pygments: http://pygments.org/
 
 LICENCE
 ---------------------------------------------
 
+Copyright (c) 2005-2012, Michele Simionato
+All rights reserved.
+
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
-met::
+met:
 
   Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.
@@ -781,18 +798,18 @@ met::
   the documentation and/or other materials provided with the
   distribution. 
 
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
-  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
-  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
-  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-  DAMAGE.
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
+TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
+USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
+DAMAGE.
 
 If you use this software and you are happy with it, consider sending me a 
 note, just to gratify my ego. On the other hand, if you use this software and
@@ -800,8 +817,8 @@ you are unhappy with it, send me a patch!
 """
 from __future__ import with_statement
 import sys, threading, time, functools, inspect, itertools  
-import multiprocessing
 from decorator import *
+from functools import partial
 from setup import VERSION
 
 today = time.strftime('%Y-%m-%d')
@@ -809,11 +826,13 @@ today = time.strftime('%Y-%m-%d')
 __doc__ = __doc__.replace('$VERSION', VERSION).replace('$DATE', today)
 
 def decorator_apply(dec, func):
-    "Decorate a function using a signature-non-preserving decorator"
-    fun = FunctionMaker(func)
-    src = '''def %(name)s(%(signature)s):
-    return decorated(%(signature)s)'''
-    return fun.make(src, dict(decorated=dec(func)), undecorated=func)
+    """
+    Decorate a function by preserving the signature even if dec 
+    is not a signature-preserving decorator.
+    """
+    return FunctionMaker.create(
+        func, 'return decorated(%(signature)s)',
+        dict(decorated=dec(func)), __wrapped__=func)
 
 def _trace(f, *args, **kw):
     print "calling %s with args %s, %s" % (f.__name__, args, kw)
@@ -878,7 +897,7 @@ def identity_dec(func):
 @identity_dec
 def example(): pass
 
-def memoize25(func):
+def memoize_uw(func):
     func.cache = {}
     def memoize(*args, **kw):
         if kw: # frozenset is used to ensure hashability
@@ -909,8 +928,8 @@ def memoize(f):
     f.cache = {}
     return decorator(_memoize, f)
 
-def blocking(not_avail="Not Available"):
-    def _blocking(f, *args, **kw):
+def blocking(not_avail):
+    def blocking(f, *args, **kw):
         if not hasattr(f, "thread"): # no thread running
             def set_result(): f.result = f(*args, **kw)
             f.thread = threading.Thread(None, set_result)
@@ -919,9 +938,9 @@ def blocking(not_avail="Not Available"):
         elif f.thread.isAlive():
             return not_avail
         else: # the thread is ended, return the stored result
-            del f.thread 
+            del f.thread
             return f.result
-    return decorator(_blocking)
+    return decorator(blocking)
 
 class User(object):
     "Will just be able to see a page"
@@ -938,73 +957,69 @@ def get_userclass():
 class PermissionError(Exception):
     pass
 
-class Restricted(object):
-    """
-    Restrict public methods and functions to a given class of users.
-    If instantiated twice with the same userclass return the same
-    object.
-    """
-    _cache = {} 
-    def __new__(cls, userclass):
-        if userclass in cls._cache:
-            return cls._cache[userclass]
-        self = cls._cache[userclass] = super(Restricted, cls).__new__(cls)
-        self.userclass = userclass
-        return self
-    def call(self, func, *args, **kw):
+def restricted(user_class):
+    def restricted(func, *args, **kw):
+        "Restrict access to a given class of users"
         userclass = get_userclass()
-        if issubclass(userclass, self.userclass):
+        if issubclass(userclass, user_class):
             return func(*args, **kw)
         else:
             raise PermissionError(
-            '%s does not have the permission to run %s!'
-            % (userclass.__name__, func.__name__))
-    def __call__(self, func):
-        return decorator(self.call, func)
+                '%s does not have the permission to run %s!'
+                % (userclass.__name__, func.__name__))
+    return decorator(restricted)
 
 class Action(object):
-    @Restricted(User)
+    """
+    >>> a = Action()
+    >>> a.view() # ok
+    >>> a.insert() # err
+    Traceback (most recent call last):
+       ...
+    PermissionError: User does not have the permission to run insert!
+
+    """
+    @restricted(User)
     def view(self):
         pass
 
-    @Restricted(PowerUser)
+    @restricted(PowerUser)
     def insert(self):
         pass
 
-    @Restricted(Admin)
+    @restricted(Admin)
     def delete(self):
         pass
-    
+
 class TailRecursive(object):
     """
     tail_recursive decorator based on Kay Schluehr's recipe
     http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/496691
+    with improvements by me and George Sakkis.
     """
-    CONTINUE = object() # sentinel
 
     def __init__(self, func):
         self.func = func
         self.firstcall = True
+        self.CONTINUE = object() # sentinel
 
     def __call__(self, *args, **kwd):
-        try:
-            if self.firstcall: # start looping
-                self.firstcall = False
-                while True:            
-                    result = self.func(*args, **kwd)
-                    if result is self.CONTINUE: # update arguments
+        CONTINUE = self.CONTINUE
+        if self.firstcall:
+            func = self.func
+            self.firstcall = False
+            try:
+                while True:
+                    result = func(*args, **kwd)
+                    if result is CONTINUE: # update arguments
                         args, kwd = self.argskwd
                     else: # last call
-                        break
-            else: # return the arguments of the tail call
-                self.argskwd = args, kwd
-                return self.CONTINUE
-        except: # reset and re-raise
-            self.firstcall = True
-            raise
-        else: # reset and exit
-            self.firstcall = True 
-            return result
+                        return result
+            finally:
+                self.firstcall = True
+        else: # return the arguments of the tail call
+            self.argskwd = args, kwd
+            return CONTINUE
 
 def tail_recursive(func):
     return decorator_apply(TailRecursive, func)
@@ -1019,5 +1034,19 @@ def fact(n): # this is not tail-recursive
     if n == 0: return 1
     return n * fact(n-1)
 
+def a_test_for_pylons():
+    """
+    In version 3.1.0 decorator(caller) returned a nameless partial
+    object, thus breaking Pylons. That must not happen again.
+
+    >>> decorator(_memoize).__name__
+    '_memoize'
+
+    Here is another bug of version 3.1.1 missing the docstring to avoid:
+
+    >>> factorial.__doc__
+    'The good old factorial'
+    """
+
 if __name__ == '__main__':
     import doctest; doctest.testmod()
